;--- http://radiohlam.ru -  ,    ---
.device ATtiny2313
.include "tn2313def.inc"
.list
;--  (   ) ---
; :    01 -   /   
;           02 -   (  )
;           03 -   (  )
; : 01 - 
;           02 -  1 
;           03 -  1 
;           04 -  1 
;           05 -  1  (0 - 0F, 1 - 8F)
;  :
;           33 - Read ROM (   1  -   + 6   +  CRC)
;           55 - Match ROM (  ,      8  ,   .       )
;           CC - Skip ROM (  8  ROM  ,   1   )
;--   
.def     w=r16           ;    
.def     Flags=r17       ;  1-wire
.def     Transmit_R=r18  ;   
.def     Recieve_R=r20   ;  
.def     Bit_counter=r21 ;  / 

;--   (   )
.equ     line_1_wire=2   ; PortB2/PinB2 -      1-wire
.equ     Port_reg=PORTB
.equ     Pin_reg=PINB
.equ     DDR_reg=DDRB
;----------------------------
.equ SEND_BIT=0
.equ READ_BIT=1
.equ SEND_RESET=2
.equ FREE_BUS=3
.equ SLAVE_IS=4
;--   
        .cseg
        .org 0
        rjmp Init        ;     ( )
;--    
;--    -  reti,  -    
        reti         ;   INT0
		reti         ;   INT1
		reti         ; Input capture interrupt 1
		reti         ; Timer/Counter1 Compare Match A
		reti         ; Overflow1 Interrupt
		reti         ; Overflow0 Interrupt
		rjmp RX_INT  ; USART0 RX Complete Interrupt
		reti         ; USART0 Data Register Empty Interrupt
		reti         ; USART0 TX Complete Interrupt
		reti         ; Analog Comparator Interrupt
		reti         ; Pin Change Interrupt
		reti         ; Timer/Counter1 Compare Match B
		reti         ; Timer/Counter0 Compare Match A
		reti         ; Timer/Counter0 Compare Match B
		reti         ; USI start interrupt
		reti         ; USI overflow interrupt
		reti         ; EEPROM write complete
		reti         ; Watchdog Timer Interrupt
;--  
Init:   ldi w,RAMEND     ;   
        out SPL,w        ;     RAM
        sbi ACSR,ACD     ;  
		;--  
        ser w            ;    w=0xFF
        out DDRA,w       ;   A.    
        out DDRD,w       ;   D.  - 
		clr w            ;    w=0x00
		out PORTA,w      ;    
        out PORTD,w
        ldi w,0b11111011 ;   B. PB2 - ,  - 
        out DDRB,w       ;
		clr w            ;        
		out PORTB,w      ; ( - ,  )
		;--  UART
		out UBRRH,w      ; UBRR (  20    115200)
		ldi w,10         ;  10, .. UBRRH=0, UBRRL=10
		out UBRRL,w
		ldi w,0b00001110 ;   USBS, UCSZ1:0
		out UCSRC,w      ; : 1 , 8 , 2 
		sbi UCSRB, TXEN  ;  
		nop
		sbi UCSRB, RXEN  ;  
		;--    
		sbi UCSRB,RXCIE
        ;--  
        clr w
        out TCCR0A,w  ;     OC0A(B)
        out TCNT0,w   ;  
        out TIMSK,w   ;     
        ser w
        out TIFR,w    ;   
		;--  ,     
		ldi w,0x01
		out UDR,w
		;--   
        sei
        ;--     ---
Wait_data:
		rjmp Wait_data
;************************************************
;------------------------------------------------
;---   UART -------------------------
RX_INT: in   w,UDR     ;      w
        cpi  w,0x01    ;    1?
		breq Reset
		cpi  w,0x02    ;    2?
		breq Send
		cpi  w,0x03    ;    3?
		breq Read
		cpi  w,0x04    ;    4?
		breq Send_One_Bit
		cpi  w,0x05    ;    5?
		breq Read_One_Bit
		;--       
Error_Command:
        ldi  w,0xFF
		out  UDR,w
		reti
;---   -------------------------
Reset:
        rcall Send_Reset_proc
		ldi   w,0x02           ;  
		sbrc  Flags,SLAVE_IS
		ldi   w,0x03           ;  
		out   UDR,w            ;   
        reti
;------------------------------
Send:   ldi w,0x01             ;  ,     
		out UDR,w
		ldi Bit_counter,8      ;   8 
Wait_byte:
        sbis UCSRA,RXC
		rjmp Wait_byte
		in   Transmit_R,UDR    ;      Transmit_R
Send_next_bit:
        rcall Send_Bit_proc
		lsr  Transmit_R
		dec  Bit_counter
		brne Send_next_bit     ;      - 
        ldi w,0x01             ;  ,    
		out UDR,w
		reti                   ;  
;------------------------------
Read:   ldi  Bit_counter,8      ;   8 
Read_next_bit:
        rcall Read_Bit_proc
		dec  Bit_counter
		brne Read_next_bit     ;      -  
		out  UDR, Recieve_R    ;   ,  
		reti                   ;  
;------------------------------
Send_One_Bit:
        ldi w,0x01             ;  ,   ,    
		out UDR,w
Wait_byte2:                    ;       
        sbis UCSRA,RXC
		rjmp Wait_byte2
		in   Transmit_R,UDR    ;      Transmit_R
        rcall Send_Bit_proc
        ldi w,0x01             ;  ,   
		out UDR,w
		reti                   ;  
;------------------------------
Read_One_Bit:
        ldi Recieve_R,0b00011110
        rcall Read_Bit_proc
		out  UDR, Recieve_R    ;   ,   (1 - 8F, 0 - 0F)
		reti                   ;  
;************************************************
;------------------------------------------------
;---  1-Wire ---------------------------
_1_wire_null:
  sbi  DDR_reg,line_1_wire
  ret
_1_wire_Z:
  cbi  DDR_reg,line_1_wire
  ret
;------------------------------------------------
Send_Reset_proc:
            sbr Flags, 1<<SEND_RESET
            cbr Flags, 1<<SLAVE_IS
          ;---   ---
            ldi w,177             ;    (255-78)
            out TCNT0,w
            ldi w,216             ; +39 
            out OCR0A,w
            ldi w, 221            ; +5 
            out OCR0B,w
          ;---    ---
            ldi w,0b00000100
            out TCCR0B,w          ;     1:256
			rcall _1_wire_null    ;     
          ;---     ---
M1:         in w,TIFR
            sbrs w,OCF0A          ;    -  
            rjmp M1
            rcall Occured_OCF0A   ;  
          ;---     ---
M2:         in w,TIFR
            sbrs w,OCF0B          ;    -  
            rjmp M2
            rcall Occured_OCF0B   ;  
          ;---     ---
M3:         in w,TIFR
            sbrs w,TOV0           ;    -  
            rjmp M3
            rcall Occured_TOV0    ;  
            cbr Flags, 1<<SEND_RESET ;    "Reset"
            ret                   ;  
Send_Bit_proc:
            sbr Flags, 1<<SEND_BIT
          ;---   ---
            ldi w,85              ;    (255-170)
            out TCNT0,w
            ldi w,89              ; +4 
            out OCR0A,w
            ldi w, 241            ; +156 
            out OCR0B,w
          ;---    ---
            ldi w,0b00000010
            out TCCR0B,w          ;     1:8
			rcall _1_wire_null    ;     
          ;---     ---
M4:         in w,TIFR
            sbrs w,OCF0A       ;    -  
            rjmp M4
            rcall Occured_OCF0A   ;  
          ;---     ---
M5:         in w,TIFR
            sbrs w,OCF0B       ;    -  
            rjmp M5
            rcall Occured_OCF0B   ;  
          ;---     ---
M6:         in w,TIFR
            sbrs w,TOV0        ;    -  
            rjmp M6
            rcall Occured_TOV0    ;  
            cbr Flags, 1<<SEND_BIT  ;    
            ret                   ;  
Read_Bit_proc:
            sbr Flags, 1<<READ_BIT
          ;---   ---
            ldi w,85              ;    (255-170)
            out TCNT0,w
            ldi w,89              ; +4 
            out OCR0A,w
            ldi w, 119            ; +34  = 14,55  (+35  = 14,95 )
            out OCR0B,w
          ;---    ---
            ldi w,0b00000010
            out TCCR0B,w          ;     1:8
			rcall _1_wire_null     ;     
          ;---     ---
M7:         in w,TIFR
            sbrs w,OCF0A       ;    -  
            rjmp M7
            rcall Occured_OCF0A   ;  
          ;---     ---
M8:         in w,TIFR
            sbrs w,OCF0B       ;    -  
            rjmp M8
            rcall Occured_OCF0B   ;  
          ;---     ---
M9:         in w,TIFR
            sbrs w,TOV0        ;    -  
            rjmp M9
            rcall Occured_TOV0    ;  
            cbr Flags, 1<<READ_BIT ;    
            ret                   ;  
;--------------------------------------------
Occured_OCF0A:
               sbrc Flags, SEND_RESET
               rjmp Send_Reset_OCR0A
			   sbrc Flags, SEND_BIT
               rjmp Send_Bit_OCR0A
               sbrc Flags, READ_BIT
               rjmp Read_Bit_OCR0A
			   ret
Send_Reset_OCR0A:   ;    Send_Reset -  
               rcall _1_wire_Z
               ret
Send_Bit_OCR0A:   ;    Send_Bit -  
               sbrc Transmit_R,0 ;    -    
               rcall _1_wire_Z   ;    -  
               ret
Read_Bit_OCR0A:   ;    Read_Bit -  
               rcall _1_wire_Z
               ret
 

Occured_OCF0B:
               sbrc Flags, SEND_RESET
               rjmp Send_Reset_OCR0B
			   sbrc Flags, SEND_BIT
               rjmp Send_Bit_OCR0B
               sbrc Flags, READ_BIT
               rjmp Read_Bit_OCR0B
               ret
Send_Reset_OCR0B:   ;    Send_Reset -  "Presence"
               sbis Pin_reg,line_1_wire  ;  1 -  
               sbr Flags, 1<<SLAVE_IS
               ret
Send_Bit_OCR0B:   ;    Send_Bit -  
               rcall _1_wire_Z
               ret
Read_Bit_OCR0B:   ;    Read_Bit -    
               lsr Recieve_R ;   ,     
               sbic Pin_reg, line_1_wire ;  0 -  
               sbr Recieve_R,1<<7
               ret


Occured_TOV0:
               sbrc Flags, SEND_RESET
               rjmp Send_Reset_TOV0
			   sbrc Flags, SEND_BIT
               rjmp Send_Bit_TOV0
               sbrc Flags, READ_BIT
               rjmp Read_Bit_TOV0
               ret
Send_Reset_TOV0:   ;    Send_Reset -  
Send_Bit_TOV0:     ;    Send_Bit -  -
Read_Bit_TOV0:     ;    Read_Bit -  -
               clr w
               out TCCR0B,w          ;  
               ser w
               out TIFR,w            ;   ,
               ret
